/* 
 * Universidade do Estado do Rio de Janeiro
 * Faculdade de Engenharia 
 * Departamento de Sistemas e Computacao
 * Autora: Elaine de M. Silva
 * Projeto de Graduacao - Adequacao do Software 
 * de Simulacao Scicos para a Aquisicao de Dados
 * e o Controle em Tempo Real
 * -------------------------------------------------------------------------------------------
 * Comedi acquisition server: main module.
 * 
 * Revision 1.1  2009/08/03 01:42:10  elaine
 * Initial revision
 *
 * Revision 1.2  2009/12/05 10:15:00  elaine
 * Minor enhancements
 *
 * Revision 1.3  2009/12/22 10:00:00  elaine
 * Recfatored code: included the sender function inside the main server loop, instead 
 * of calling it from the execComedi function.  
 * Added command line option to process log file. Calls to ./acqd -v writes to log file.  
 * 
 *
 * -------------------------------------------------------------------------------------------
 *
 */


/* required header files */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <stddef.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <netinet/tcp.h>
#include <comedilib.h>  // provides comedi functions
#include <time.h>
#include <getopt.h>
#include "includes/comedisrv.h" // provides comedi modified functions and constants

int verbose =0;

// displays the help menu
static void usage(){
	fputs("Usage: acqd [-h| --help] [-v| --verbose]\n",stderr);
	fputs("\t-h (or --help) \t Displays this help screen. \n",stderr);
	fputs("\t-v (or --verbose) \t Saves log information on the 'log.txt' file.\n",stderr);
}

struct option options[] = {
	{"verbose",0,NULL,'v'},
	{"help",0,NULL,'h'},
	{0},
};

const char *comediErr; //used on the execComedi function. Holds errors thrown by the comedi functions.


int main(int argc, char **argv ){

	/* variables */
	comediErr = (char *)malloc(BUFFERSIZE);
	int sockfd = -1 , newSockfd = -1;               // initialize sockets
	int stat, sentBytes;                            // return code
	char buffer[BUFFERSIZE];                        // message buffer
	char *sendStatus = (char *) malloc(BUFFERSIZE); // messages received from client
        char *onStatus = (char *) malloc(BUFFERSIZE);   // status received from client
        char *params[MAX_PARAMS];                       // parameter buffer
	char IPv6String[INET6_ADDRSTRLEN];              // space to hold the IPv6 string
	char tokensFound = 0;                           // counts received parameters
	struct sockaddr_in clientAddr;                 // large enough to hold IPv4 or 6 addresses
	time_t td;
	socklen_t sin_size;
        int argOpt, index;
	char *logfile = "/dev/null";                    // initialize logfile


	/* treats command line arguments */
	while (1){

		argOpt = getopt_long(argc,argv,"vh",options,&index);
		if (argOpt == -1)
			break;
		switch(argOpt){
			case 'v':
				verbose = 1;
				puts("Initiating verbose mode.\n");
				break;
			case 'h':
				usage();
				return 1;
							
			default:
				puts("Use --help for help.\n");
				return 1;
		}
	
	}//end of the while loop

	if(verbose){
		logfile = "log.txt";
	}
	


	/* intitialize server log file */
	FILE *log = fopen(logfile,"a");  
	if (!log){
		fprintf(stderr, "%s: server log file failure: %s.\n",strerror(errno),logfile);
		exit(13);
	}



	/* initialize parameter and message buffers */ 
	bzero(params, MAX_PARAMS*sizeof(char));  
	bzero(buffer, BUFFERSIZE*sizeof(char));  

	if ((sockfd = getLocalSocket(SERVER_PORT)) < 0){
		fprintf(log,"acqd: error trying to bind\n");
		fflush(log);
		exit(1);
	}
	
        // binds the server socket to SERVER_PORT
	stat = listen(sockfd, BACKLOG); // waiting for incoming connections on port SERVER_PORT. On failure, restarts. 
	if (stat < 0){
		perror("listen");
		exit(1);
	}

	printf("acqd: waiting calls...\n");

	sin_size = sizeof(struct sockaddr_in);

	for(;;){

		newSockfd = accept(sockfd, (struct sockaddr *)&clientAddr, &sin_size); // accepts connection requests. On failure restarts
		if(newSockfd == -1){ 
			perror("accept");
			continue;
			}

		int nodelay = 1;
		setsockopt(newSockfd,SOL_TCP,TCP_NODELAY,&nodelay,sizeof(nodelay));
		
		// converts numeric address in a string (for printing purposes)
		inet_ntop(clientAddr.sin_family,getInAddr((struct sockaddr *)&clientAddr),IPv6String,sizeof IPv6String);
		fprintf(log,"acqd: connected to %s\n",IPv6String);
		fflush(log);

		// receives client messages and stores on buffer. On failure closes new socket and restarts.
		if((stat = getClientData(sockfd,newSockfd,buffer,BUFFERSIZE)) < 0){
			fprintf(log,"acqd: no data received\n");
			fflush(log);
			close(newSockfd);
			continue;			
		}

		// prints log file with incoming messages
		time(&td);
		fprintf(log,"IN~%s~%s~%s\n",IPv6String, ctime(&td), buffer);
		fflush(log);

		// treats received messages and prepares execution of comedi functions 
		tokensFound = parser(buffer, MAX_PARAMS,params);
		if(tokensFound < 0){
			if(tokensFound == -1){
				fprintf(log,"acqd: invalid string. Try '<IP>/comedi_function/par1/par2/par3...'\n");
				fflush(log);
				continue;
			}

			if(tokensFound == -2){
				fprintf(log,"acqd: first argument of sent message must be GET\n");
				fflush(log);
				continue;
			}
			
			if(tokensFound == -3){
				fprintf(log,"acqd: there must be a valid comedi function\n");
				fflush(log);
				continue;
			}
		}
		else{
			//executes comedi function
			execComedi(newSockfd, params,sendStatus,onStatus);

			//sends messages to client
			sentBytes = sendMsg(onStatus,sendStatus,newSockfd,strlen(sendStatus));
			if (sentBytes < 0){
				perror("acqd: write");
			}else{

				// prints log file with outcoming messages
				time(&td);
				fprintf(log,"OUT~%s~%s\n",ctime(&td),sendStatus);
				fflush(log);

			}
		}

	}//end of the for loop (main server loop)

	close(sockfd);
	free(onStatus);
	free(sendStatus);
	fclose(log);

	return 0;


	
}

